/***************************************************************************
                          config.c  -  description
                             -------------------
    begin                : Tue Feb 13 2001
    copyright            : (C) 2001 by Michael Speck
    email                : kulkanie@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <SDL.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "sdl.h"
#include "config.h"

Config config;

#define CONFIG_FILE_NAME "ltris.conf"

/* game type names. hacky place to define them but who cares? */
char *gametype_names[8] = {
    "Demo",
    "Classic",
    "Figures",
    "Vs Human",
    "Vs CPU",
    "Vs Human&Human",
    "Vs Human&CPU",
    "Vs CPU&CPU"
};

/* check if config directory exists; if not create it and set config_dir */
void config_check_dir()
{
    sprintf( config.dir_name, "%s/%s", getenv( "HOME" ), CONFIG_DIR_NAME );
    if ( opendir( config.dir_name ) == 0 ) {
        fprintf( stderr, "couldn't find/open config directory '%s'\n", config.dir_name );
        fprintf( stderr, "attempting to create it... " );
        mkdir( config.dir_name, S_IRWXU );
        if ( opendir( config.dir_name ) == 0 )
            fprintf( stderr, "failed\n" );
        else
            fprintf( stderr, "ok\n" );
    }
}

/* set config to default */
void config_reset()
{
    /* game options */
    config.gametype = GAME_DEMO;
    config.starting_level = 0;
    config.preview = 1;
	config.help = 0;
	config.slow = 0;
    strcpy( config.player1.name, "Michael" );
    strcpy( config.player2.name, "Sabine" );
    strcpy( config.player3.name, "Thomas" );
    /* multiplayer */
    config.wholes = 2;
    config.send_all = 0;
    config.send_tetris = 1;
    /* cpu */
    config.cpu_aggr = 1;
    config.cpu_delay = 1000;
    config.cpu_rot_delay = 100;
    /* controls */
    config.hori_delay = 3;
    config.player1.controls.left = SDLK_LEFT;
    config.player1.controls.right = SDLK_RIGHT;
    config.player1.controls.rot_left = SDLK_UP;
    config.player1.controls.rot_right = SDLK_PAGEDOWN;
    config.player1.controls.down = SDLK_DOWN;
    config.player1.controls.drop = SDLK_END;
    config.player2.controls.left = 'a';
    config.player2.controls.right = 'd';
    config.player2.controls.rot_left = 'w';
    config.player2.controls.rot_right = 'e';
    config.player2.controls.down = 's';
    config.player2.controls.drop = 'y';
    config.player3.controls.left = SDLK_KP1;
    config.player3.controls.right = SDLK_KP3;
    config.player3.controls.rot_left = SDLK_KP5;
    config.player3.controls.rot_right = SDLK_KP6;
    config.player3.controls.down = SDLK_KP2;
    config.player3.controls.drop = SDLK_KP8;
    /* sound */
    config.sound = 1;
    config.volume = 6; /* 1 - 8 */
    /* graphics */
    config.trp = 1;
    config.anim = 1;
    config.fullscreen = 0;
    config.fade = 1;
    config.fps = 0; /* frames per second: 0 - no limit, 1 - 50, 2 - 100, 3 - 200 */
	config.bkgnd = 0;
	config.smooth_hori = 1;
	config.block_by_block = 0;
    /* lbreakout2 event data */
    config.rel_motion = 0;
    config.motion_mod = 100;
    config.invert = 0;
    config.grab = 0;
    /* various */
    config.quick_help = 1;
    config.visualize = 0;
    config.keep_bkgnd = 0;
    config.clear_keystate = 1;
    config.center_preview = 0;
}

/* load config */
void config_load( )
{
    FILE *file = 0;
    char file_name[512];
    struct stat dir_stat;
    char dir_name[512];

    /* check if directory exists, if not create it */
    config_check_dir();

    sprintf( file_name, "%s/%s", config.dir_name, CONFIG_FILE_NAME );
    if ( ( file = fopen( file_name, "r" ) ) == 0 ) {
        /* not found? use defaults! */
        fprintf( stderr, "config file '%s' not found... using defaults\n", file_name );
        config_reset();
    }
    else {
        /* check if file size fits struct size; if not use defaults */
        stat( file_name, &dir_stat );
        if ( dir_stat.st_size != sizeof( Config ) ) {
            fprintf( stderr, "config file '%s' corrupted... using defaults\n", file_name );
            config_reset();
        }
        else {
            /* must backup dir_name! */
            strcpy( dir_name, config.dir_name );
            /* read it from file */
            fread( &config, sizeof( Config ), 1, file );
            strcpy( config.dir_name, dir_name );
        }
        fclose( file );
    }
}

/* save config */
void config_save( )
{
    FILE *file = 0;
    char file_name[512];

    sprintf( file_name, "%s/%s", config.dir_name, CONFIG_FILE_NAME );
    if ( ( file = fopen( file_name, "w" ) ) == 0 )
        fprintf( stderr, "cannot access config file '%s' to save settings\n", file_name );
    else {
        fwrite( &config, sizeof( Config ), 1, file );
        fclose( file );
    }
}

